@@ -1,9 +1,9 @@
 from collections import defaultdict
-from typing import Any, Callable, DefaultDict, List, Optional, Tuple, Dict, TypeVar
-from PIL import Image, ImageDraw, ImageFont
+from typing import (Any, Callable, DefaultDict, Generator, List, Optional,
+                    Tuple, Dict, TypeVar)
+from PIL import Image, ImageDraw
 import numpy as np
 from math import pi
-from pprint import pprint
 import tatsu
 from tatsu.walkers import NodeWalker
 import json
@@ -11,7 +11,6 @@ import sys
 import os
 
 from importlib import resources
-import io
 
 Color = str
 TileColor = Tuple[Optional[Color], Optional[Color]]
@@ -24,12 +23,15 @@ S = TypeVar("S")
 # Error
 ###############################################################################
 
+
 class MapperError(Exception):
     pass
 
+
 class MapperRuntimeError(MapperError):
     pass
 
+
 class MapperCompilationError(MapperError):
     pass
 
@@ -68,10 +70,11 @@ def tint(a: Tuple[float, ...]) -> Tuple[int, ...]:
 
 
 def tran_position(pos: Tuple[int, int], dims: Tuple[int, int],
-                  tile_width: int, center: Tuple[int, int]) -> Tuple[int, int]:
-    vals = tadd(tmul(tsub(pos, center), (tile_width, tile_width)),
-                tint(tdiv(dims, (2, 2))))
-    return vals[0], vals[1]
+                  tile_width: int, center: Tuple[float, float]) \
+        -> Tuple[float, float]:
+    x = ((pos[0] - center[0]) * tile_width + dims[0] // 2)
+    y = ((pos[1] - center[1]) * tile_width + dims[1] // 2)
+    return x, y
 
 ###############################################################################
 # MAP AND SHIP
@@ -100,7 +103,8 @@ class Map:
         return self.tiles[tile]
 
     def put(self, tile: Tuple[int, int],
-            color: Optional[Color] = None, border_color: Optional[Color] = None):
+            color: Optional[Color] = None,
+            border_color: Optional[Color] = None) -> None:
 
         (new_color, new_border) = self.tiles[tile]
         if color is not None:
@@ -109,14 +113,15 @@ class Map:
             new_border = border_color
         self.tiles[tile] = (new_color, new_border)
 
-    def get_tiles(self):
+    def get_tiles(self) -> Generator[Tuple[Tuple[int, int], TileColor],
+                                     None, None]:
         yield from self.tiles.items()
         yield self.start, (None, 'green')
 
-    def reset(self):
+    def reset(self) -> None:
         self.tiles.clear()
 
-    def shift(self, dx, dy):
+    def shift(self, dx: int, dy: int) -> None:
         temp_tiles = {}
         for (x, y), color in self.tiles.items():
             temp_tiles[(x + dx, y + dy)] = color
@@ -180,7 +185,7 @@ class Ship:
         self.pendown = True
         self.color = 'black'
 
-    def move_fwd(self, amount) -> 'Ship':
+    def move_fwd(self, amount: int) -> 'Ship':
         if self.pendown:
             self.put(self.color)
 
@@ -205,7 +210,7 @@ class Ship:
         return self
 
     def put(self, color: Optional[Color] = None,
-            border_color: Optional[Color] = None):
+            border_color: Optional[Color] = None) -> None:
         self.mapp.put(self.position, color=color, border_color=border_color)
 
 
@@ -261,7 +266,7 @@ def fill_from_file(mapp: Map, path: str, sep: Optional[str] = None) -> Map:
         return(fill_from_text(mapp, f.read(), sep))
 
 
-def export_map(mapp: Map, sep:str = '\n') -> str:
+def export_map(mapp: Map, sep: str = '\n') -> str:
     res = 'LIST'
     for (col, row), (color, _) in mapp.tiles.items():
         if color is not None:
@@ -288,7 +293,7 @@ def scale_matrix(scale: float) -> np.ndarray:
 
 
 def transform(polygon: np.ndarray, pos: Tuple[int, int],
-              rot: float, scale: float):
+              rot: float, scale: float) -> np.ndarray:
 
     t, r, s = translation_matrix(
         pos), rotation_matrix(rot), scale_matrix(scale)
@@ -302,15 +307,17 @@ def transform(polygon: np.ndarray, pos: Tuple[int, int],
 SHIP = np.array(((0, -1, 1), (1, 1, 1), (-1, 1, 1))).T
 
 
-def render_map(path: str, mapp: Map, ship: Optional[Ship] = None):
+def render_map(path: str, mapp: Map, ship: Optional[Ship] = None) -> None:
     im = draw_map(mapp, ship)
     im.save(path)
 
-def show_map(mapp: Map, ship: Optional[Ship] = None):
+
+def show_map(mapp: Map, ship: Optional[Ship] = None) -> None:
     im = draw_map(mapp, ship)
     im.show()
 
-def draw_map(mapp: Map, ship: Optional[Ship] = None):
+
+def draw_map(mapp: Map, ship: Optional[Ship] = None) -> Image.Image:
     width = 800
     height = 500
 
@@ -320,13 +327,13 @@ def draw_map(mapp: Map, ship: Optional[Ship] = None):
     tile_width = 50
 
     if right != cx:
-        tile_width = min(tile_width, width / 2 / (right - cx + 1))
+        tile_width = int(min(tile_width, width / 2 / (right - cx + 1)))
     if left != cx:
-        tile_width = min(tile_width, width / 2 / (cx - left + 1))
+        tile_width = int(min(tile_width, width / 2 / (cx - left + 1)))
     if bottom != cy:
-        tile_width = min(tile_width, height / 2 / (bottom - cy + 1))
+        tile_width = int(min(tile_width, height / 2 / (bottom - cy + 1)))
     if top != cy:
-        tile_width = min(tile_width, height / 2 / (cy - top + 1))
+        tile_width = int(min(tile_width, height / 2 / (cy - top + 1)))
 
     tile_width = int(tile_width)
 
@@ -337,17 +344,18 @@ def draw_map(mapp: Map, ship: Optional[Ship] = None):
     g = ImageDraw.Draw(im)
 
     for pos, (color, border_color) in mapp.get_tiles():
-        coord = tsub(tran_position(pos, dims, tile_width, center), #TODO center is float float
-                        (tile_width // 2, tile_width // 2))
+        coord = tran_position(pos, dims, tile_width, center)
+        coord = coord[0] - tile_width // 2, coord[1] - tile_width // 2
         g.rectangle(
-            [coord, tadd(coord, (tile_width, tile_width))],
+            [coord, (coord[0] + tile_width, coord[1] + tile_width)],
             fill=color, width=tile_width//10,
             outline=border_color)
 
     if ship is not None:
         rot = ship.direction * pi / 2
-        pos = tran_position(ship.position, dims, tile_width, center)
-        polygon = transform(SHIP, pos, rot, tile_width / 3).T.tolist()
+        ship_pos = tran_position(ship.position, dims, tile_width, center)
+        polygon = transform(SHIP, (int(ship_pos[0]), int(ship_pos[1])), rot,
+                            tile_width / 3).T.tolist()
         res_poly = tuple([tuple(p[:2]) for p in polygon])
         g.polygon(tuple(res_poly), "gray")
 
@@ -358,7 +366,10 @@ def draw_map(mapp: Map, ship: Optional[Ship] = None):
 # PARSING
 ###############################################################################
 
-def mapper_parser(asmodel=True):
+MapperParser = Any
+
+
+def mapper_parser(asmodel: bool = True) -> MapperParser:
     with resources.open_text('ellatu', 'mapper.ebnf') as f:
         grammar = f.read()
         return tatsu.compile(grammar, asmodel=asmodel)
@@ -413,7 +424,7 @@ class ScopeStack:
     def __init__(self, stack: Optional[List[Dict[str, Any]]] = None):
         self.stack: List[Dict[str, Any]] = stack if stack is not None else []
 
-    def lookup(self, symbol: str):
+    def lookup(self, symbol: str) -> Optional[Any]:
         for dic in reversed(self.stack):
             if symbol in dic:
                 return dic[symbol]
@@ -432,17 +443,17 @@ class ScopeStack:
         assert self.stack
         self.stack[-1][symbol] = value
 
-    def add_scope(self):
+    def add_scope(self) -> None:
         self.stack.append({})
 
-    def pop_scope(self):
+    def pop_scope(self) -> None:
         self.stack.pop()
 
-    def copy(self, layers: int = 1):
+    def copy(self, layers: int = 1) -> 'ScopeStack':
         return ScopeStack(self.stack[:layers])
 
 
-ARIT_OPS = {
+ARIT_OPS: Dict[str, Callable[[int, int], int]] = {
     '&&': lambda a, b: int(a and b),
     '||': lambda a, b: int(a or b),
     '=>': lambda a, b: int(a and not b),
@@ -462,12 +473,14 @@ ARIT_OPS = {
     '%': lambda a, b: a % b,
 }
 
+MapperASTNode = Any
+
 
 class MapperWalker(NodeWalker):
 
     def __init__(self, glob_scope: ScopeStack,
-                 inbuilt: Optional[Dict[str,
-                                        Callable[['MapperWalker', List[Any]], Any]]],
+                 inbuilt: Optional[
+                     Dict[str, Callable[['MapperWalker', List[Any]], Any]]],
                  ship: Ship,
                  mapp: Map) \
             -> None:
@@ -477,18 +490,18 @@ class MapperWalker(NodeWalker):
         self.ship = ship
         self.mapp = mapp
 
-    def walk_object(self, node):
+    def walk_object(self, node: MapperASTNode) -> MapperASTNode:
         return node
 
-    def walk__function(self, node):
+    def walk__function(self, node: MapperASTNode) -> MapperASTNode:
         self.scope.put(node.name, node)
         return node
 
-    def walk__amn_function(self, node):
+    def walk__amn_function(self, node: MapperASTNode) -> MapperASTNode:
         self.walk(node.fun)
         return None
 
-    def walk__codeblock(self, node):
+    def walk__codeblock(self, node: MapperASTNode) -> MapperASTNode:
         self.scope.add_scope()
         value = None
         for line in node.lines:
@@ -498,7 +511,7 @@ class MapperWalker(NodeWalker):
         self.scope.pop_scope()
         return value
 
-    def walk__if_statement(self, node):
+    def walk__if_statement(self, node: MapperASTNode) -> MapperASTNode:
         # print('if')
 
         if self.walk(node.cond):
@@ -513,7 +526,7 @@ class MapperWalker(NodeWalker):
 
         return None
 
-    def walk__while_stmt(self, node):
+    def walk__while_stmt(self, node: MapperASTNode) -> MapperASTNode:
 
         while self.walk(node.cond):
             result = self.walk(node.body)
@@ -528,7 +541,7 @@ class MapperWalker(NodeWalker):
                 return result
         return None
 
-    def walk__for_stmt(self, node):
+    def walk__for_stmt(self, node: MapperASTNode) -> MapperASTNode:
 
         self.scope.add_scope()
         gen = self.walk(node.gen)
@@ -559,14 +572,14 @@ class MapperWalker(NodeWalker):
         self.scope.pop_scope()
         return return_value
 
-    def walk__lift(self, node):
+    def walk__lift(self, node: MapperASTNode) -> MapperASTNode:
         return (node.lift, self.walk(node.value))
 
-    def walk__amnesia_stmt(self, node):
+    def walk__amnesia_stmt(self, node: MapperASTNode) -> MapperASTNode:
         self.walk(node.stmt)
         return None
 
-    def walk__call(self, node):
+    def walk__call(self, node: MapperASTNode) -> MapperASTNode:
 
         walked_args = [self.walk(v) for v in node.args]
 
@@ -580,7 +593,7 @@ class MapperWalker(NodeWalker):
 
         if len(function.params) != (len(node.args)):
             raise MapperRuntimeError(
-                f"Invalid number of positional arguments, function " +
+                "Invalid number of positzonal arguments, function " +
                 f"'{node.name}' takes {len(function.params)} arguments, " +
                 f"but {node.args} were given.")
 
@@ -604,38 +617,46 @@ class MapperWalker(NodeWalker):
                 f"A uncaught '{lift}' lift tried to escape function")
         return value
 
-    def walk__neg(self, node):
+    def walk__neg(self, node: MapperASTNode) -> MapperASTNode:
         if node.op == '!':
             return int(not self.walk(node.val))
         elif node.op == '-':
             return - self.walk(node.val)
         raise MapperRuntimeError(f"Invalid negation operator '{node.op}'")
 
-    def walk__range(self, node):
+    def walk__range(self, node: MapperASTNode) -> MapperASTNode:
         start = self.walk(node.start)
         end = self.walk(node.end)
         offset = (1 if end >= start else -1)
-
         step = self.walk(node.step) if node.step is not None else offset
+
+        if (not isinstance(start, int) or not isinstance(end, int)
+                or not isinstance(step, int)):
+            raise MapperRuntimeError("Values in range were not of type int")
         return range(start, end + offset, step)
 
-    def walk__assigment(self, node):
+    def walk__assigment(self, node: MapperASTNode) -> MapperASTNode:
         value = self.walk(node.value)
         self.scope.put(node.name, value)
         return value
 
-    def walk__operation(self, node):
+    def walk__operation(self, node: MapperASTNode) -> MapperASTNode:
         if node.op not in ARIT_OPS:
             raise MapperRuntimeError(f"Unknown operation '{node.op}")
-        return ARIT_OPS[node.op](self.walk(node.left), self.walk(node.right))
+        left_value = self.walk(node.left)
+        right_value = self.walk(node.right)
+        if not isinstance(left_value, int) or not isinstance(right_value, int):
+            raise MapperRuntimeError("Values in arithmetic operation " +
+                                     "were not of type int")
+        return ARIT_OPS[node.op](left_value, right_value)
 
-    def walk__variable(self, node):
+    def walk__variable(self, node: MapperASTNode) -> MapperASTNode:
         value = self.scope.lookup(node.name)
         if value is None:
             raise MapperRuntimeError(f"Variable '{node.name}' is undefined")
         return value
 
-    def walk__integer(self, node):
+    def walk__integer(self, node: MapperASTNode) -> MapperASTNode:
         return int(node.value)
 
 
@@ -649,8 +670,9 @@ def in_opt_range(value: int,
     return (left is None or left <= value) \
         and (right is None or right >= value)
 
+
 def opt_range_str(rng: Optional[Tuple[Optional[int], Optional[int]]]) \
-    -> Tuple[str, bool]:
+        -> Tuple[str, bool]:
     if rng is None:
         return 'unlimited number of', True
 
@@ -665,33 +687,37 @@ def opt_range_str(rng: Optional[Tuple[Optional[int], Optional[int]]]) \
     return f'from {bottom} to {top}', True
 
 
+MapperBuiltInFunction = Callable[[MapperWalker, List[Any]], Any]
+
+
 def argument_check(name: str, number: Optional[Tuple[Optional[int],
-                                          Optional[int]]] = None):
-    def dec(func):
-        def wrapper(ctx: MapperWalker, args: List[Any]):
+                                                     Optional[int]]] = None) \
+        -> Callable[[MapperBuiltInFunction], MapperBuiltInFunction]:
+    def dec(func: MapperBuiltInFunction) -> MapperBuiltInFunction:
+        def wrapper(ctx: MapperWalker, args: List[Any]) -> Any:
             if not in_opt_range(len(args), number):
                 rng_string, rng_pl = opt_range_str(number)
                 raise MapperRuntimeError(
                     f"Function '{name}' takes {rng_string} positional " +
                     f"argument{'s' if rng_pl else ''} " +
                     f"but {len(args)} were given.")
-            func(ctx, args)
+            return func(ctx, args)
         return wrapper
     return dec
 
 
 @argument_check('print', number=(1, None))
-def map_print(_: MapperWalker, args: List[Any]):
+def map_print(_: MapperWalker, args: List[Any]) -> None:
     print(*args)
 
 
 @argument_check('mov', number=(1, 1))
-def mov(mapper: MapperWalker, args: List[Any]):
+def mov(mapper: MapperWalker, args: List[Any]) -> None:
     mapper.ship.move_fwd(args[0])
 
 
 @argument_check('rot', number=(1, 1))
-def rot(mapper: MapperWalker, args: List[Any]):
+def rot(mapper: MapperWalker, args: List[Any]) -> None:
     mapper.ship.rotate(args[0])
 
 
@@ -699,17 +725,17 @@ COLS = ['black', 'red', 'green', 'blue']
 
 
 @argument_check('put', number=(1, 1))
-def put(mapper: MapperWalker, args: List[Any]):
+def put(mapper: MapperWalker, args: List[Any]) -> None:
     mapper.ship.put(color=COLS[args[0]])
 
 
 @argument_check('pen', number=(1, 1))
-def pen(mapper: MapperWalker, args: List[Any]):
+def pen(mapper: MapperWalker, args: List[Any]) -> None:
     mapper.ship.pendown = bool(args[0])
 
 
 @argument_check('col', number=(1, 1))
-def col(mapper: MapperWalker, args: List[Any]):
+def col(mapper: MapperWalker, args: List[Any]) -> None:
     mapper.ship.color = COLS[args[0]]
 
 
@@ -725,7 +751,7 @@ INBUILT = {
 Model = Any
 
 
-def compile_code(code: str, parser = None) -> Model:
+def compile_code(code: str, parser: Optional[MapperParser] = None) -> Model:
     if parser is None:
         parser = mapper_parser(True)
     processed = mapper_preprocessor(code)
@@ -735,10 +761,18 @@ def compile_code(code: str, parser = None) -> Model:
         raise MapperCompilationError(str(e))
 
 
-def compile_codeblocks(codeblocks: List[str], parser = None) -> List[Model]:
+def compile_codeblocks(codeblocks: List[str],
+                       parser: Optional[MapperParser] = None) -> List[Model]:
     return [compile_code(b, parser) for b in codeblocks]
 
 
+def contains_main(model: Model) -> bool:
+    for dec in model['decls']:
+        if dec.name == 'main':
+            return True
+    return False
+
+
 def run_models(models: List[Model]) -> Tuple[Any, Map, Ship]:
     mapper_map = Map(start=(0, 0))
     ship = Ship(mapper_map, mapper_map.start)
@@ -762,7 +796,8 @@ def run_models(models: List[Model]) -> Tuple[Any, Map, Ship]:
 
     return result, mapper_map, ship
 
-def generate_level(folder: str, src: str, name: str):
+
+def generate_level(folder: str, src: str, name: str) -> None:
 
     src = os.path.join(folder, src)
     org_name = name
@@ -776,7 +811,7 @@ def generate_level(folder: str, src: str, name: str):
             f.write(code)
 
     models = [compile_code(code)]
-    _ , mapp, ship = run_models(models)
+    _, mapp, ship = run_models(models)
 
     tiles = export_map(mapp, sep=';')
     with open(f"{name}.txt", "w") as f:
@@ -787,7 +822,8 @@ def generate_level(folder: str, src: str, name: str):
     render_map(f"{name}-res.png", mapp, ship)
     save_json(name, org_name, tiles)
 
-def save_json(path_name: str, org_name: str, test: str):
+
+def save_json(path_name: str, org_name: str, test: str) -> None:
 
     filepath = f"{path_name}.json"
     if os.path.isfile(filepath):
@@ -819,8 +855,7 @@ def save_json(path_name: str, org_name: str, test: str):
     with open(filepath, "w") as f:
         f.write(json_str)
 
+
 if __name__ == "__main__":
     name = sys.argv[1] if len(sys.argv) < 3 else sys.argv[2]
     generate_level('mapper', sys.argv[1], name)
-
-
